;=========================================================================
; TestExport.bb
; by ATEK Nicolas aka Kochon
;=========================================================================

Include "..\Include\KaTE library.bb" 
Graphics3D 800, 600

; Camera and light stuff
;-------------------------------------------------------------------------
Global cam = CreateCamera()
CameraRange cam, 0.1, 1024
CameraClsColor cam, 103, 106, 130
CameraFogMode cam, 1
CameraFogColor cam, 103, 106, 130
CameraFogRange cam, 64, 128

AmbientLight 64, 64, 64
Global light = CreateLight(1)
RotateEntity light, 45, 5, 0

; Layers stuff
;-------------------------------------------------------------------------
Global terrain = kCreateTerrainLayer("example.b3d", 1) ;Load a Terrain layer and set its type to 1 (for collisions)
Global layer1 = kCreateLayer() ;User layer 
Global layer2 = kCreateLayer() ;WaterPlane layer 

EntityParent kwplay(terrain), klay(layer2) ;Reparent WaterPlane from Terrain layer to WaterPlane layer (layer2) 
EntityAlpha kwplay(layer2), 0.3 ;Set a good alpha value for the WaterPlane
MoveEntity kwplay(layer2), 0, 1, 0 ;Move the WaterPlane entity to cover a little portion of the terrain

Global actor = CreatePivot(klay(layer1)) ;Create a pivot for the player and parent it to the User layer (layer1)
EntityType actor, 2 ;Set the actors collision type to 2
EntityRadius actor, 1; Set the sphere radius for actors collision

Global campiv = CreatePivot(actor) ;Attach a pivot to the actor (not the camera directly to avoid incompatibilities)
MoveEntity campiv, 0, 1, 0 ;Set the campiv 1 unit on top of the actor
MoveEntity actor, 0, 48, 0 ;Set the actors height far from the terrain

; Adding some balls  
;-------------------------------------------------------------------------
Dim balls(8)
Local n
For n = 1 To 8
	balls(n) = CreateSphere(8, klay(layer1)) ;Balls are part of layer1
	EntityColor balls(n), Rnd(64, 128), Rnd(64, 128), Rnd(64, 128)
	PositionEntity balls(n), Rnd(-24, 24), Rnd(48, 60), Rnd(-24, 24) ;Randomly position balls far higher the terrain
	EntityType balls(n), 3 ;Set balls collision type to 3
	EntityRadius balls(n), 1 ;Set the balls radius for collision
Next 

; Collisions table
;-------------------------------------------------------------------------
Collisions 2, 1, 2, 3 ; Collisions between actor and terrain
Collisions 3, 1, 2, 2 ; Collisions between balls and terrain
Collisions 3, 3, 2, 2 ; Collisions between balls
Collisions 2, 3, 2, 2 ; Collisions between actor and balls

; Mouse initialisation
;-------------------------------------------------------------------------
Global mxs#, mys# ;Values to track the mouse speed in both directions 
MoveMouse(400,300)
MouseXSpeed()
MouseYSpeed()
HidePointer()

; Main loop
;-------------------------------------------------------------------------
While Not KeyDown(1)
	UpdateActor
	UpdateBalls
	UpdateWorld
	
	;Set camera global position and rotation to the campiv entity
	PositionEntity cam, EntityX(campiv, True), EntityY(campiv, True), EntityZ(campiv, True)
	RotateEntity cam, EntityPitch(campiv, True), EntityYaw(campiv, True), EntityRoll(campiv, True)
	
	Render
	
	Text 0, 0, "FPS: " + kfraps() ; Show the FrameRate
	Flip
Wend

; Render all the layers in a good order
;-------------------------------------------------------------------------
Function Render()
	KRenderLayer cam, terrain, True, True
	KRenderLayer cam, layer1, False, False
	KRenderLayer cam, layer2, False, False
End Function

; Actor and campiv moves
;-------------------------------------------------------------------------
Function UpdateActor()
	mxs = MouseXSpeed()
	mys = MouseYSpeed()
	MoveMouse(400,300)
	
	MoveEntity actor, 0, -0.1, 0 ;Actors gravity
	
	mxs = mxs * 0.25
	mys = mys * 0.25
	If EntityPitch(campiv) + mys < -80 Then mys = -80 - EntityPitch(campiv) ;Constrain campiv to min pitch = -80 
	If EntityPitch(campiv) + mys > 80 Then mys = 80 - EntityPitch(campiv) ;Constrain campiv to max pitch = 80
	
	TurnEntity actor, 0, -mxs, 0
	TurnEntity campiv, mys, 0, 0
	
	If KeyDown(203) Then MoveEntity actor, -0.1, 0, 0 ;Move left 
	If KeyDown(205) Then MoveEntity actor, 0.1, 0, 0 ;Move right
	If KeyDown(200) Then MoveEntity actor, 0, 0, 0.1 ;Move front
	If KeyDown(208) Then MoveEntity actor, 0, 0, -0.1; Move back
End Function

; Balls moves 
;-------------------------------------------------------------------------
Function UpdateBalls()
	Local n
	For n = 1 To 8
		MoveEntity balls(n), 0, -0.25, 0
	Next
End Function
;~IDEal Editor Parameters:
;~C#Blitz3D