;=========================================================================
; KaTE library v1.0
; by ATEK Nicolas aka Kochon
;=========================================================================

Type KLS
	Field entity
End Type

Type KFPS
	Field cnt, mil, tmp 
End Type

Global fraps.KFPS = New KFPS

; kfraps(): Update and return the number of frames per second
; Need to be called once before each flip command
;-------------------------------------------------------------------------
Function kfraps()
	Local mil = MilliSecs()
	If mil > fraps\mil Then
		fraps\cnt = fraps\tmp : fraps\mil = mil + 1000 : fraps\tmp = 0
	Else
		fraps\tmp = fraps\tmp + 1
	End If
	Return fraps\cnt
End Function

; kCreateLayer(): Create a new empty layer
; Then you can parent your entities using the klay(...) command
; to retrieve the layer pivot
;-------------------------------------------------------------------------
Function kCreateLayer()
	Local layer.KLS
	layer = New KLS
	layer\entity = CreatePivot()
	Return Handle(layer)
End Function

; kFreeLayer(hlayer): Destroy a layer, its pivot and so all entities
; parented to it
;-------------------------------------------------------------------------
Function kFreeLayer(hlayer)
	Local layer.KLS
	layer = Object.KLS(hlayer)
	FreeEntity layer\entity
	Delete layer
End Function

; klay(hlayer): Return the layer pivot
; Necessary to parent entities to a layer or, a layer to an entity
; or pivot 
;-------------------------------------------------------------------------
Function klay(hlayer)
	Local layer.KLS
	layer = Object.KLS(hlayer)
	If layer = Null Then Return 0 Else Return layer\entity
End Function

; kCreateTerrainLayer(b3dfile$, ctype = 0): Load a KaTE terrain
; and parent it to a layer
; ctype: Define the type used in collisions
;-------------------------------------------------------------------------
Function kCreateTerrainLayer(b3dfile$, ctype = 0)
	Local terrain, layer.KLS, child, n
	terrain = LoadAnimMesh(b3dfile$)
	If terrain = 0 Then RuntimeError("kCreateTerrainLayer: Wrong or missing file!")
	If Not(EntityName(terrain) = "KaTE terrain") Then RuntimeError("kCreateTerrainLayer: Wrong or missing file!")
	If ctype > 0 Then
		child = FindChild(terrain, "Layer1")
		For n = 1 To CountChildren(child)
			EntityType(GetChild(child, n), ctype)
		Next
	EndIf
	layer = New KLS
	layer\entity = terrain
	Return Handle(layer)
End Function

; ksbstate(hlayer, state = -1): Set the SkyBox state of the
; Terrain layer or other layer if SkyBox has been moved to it.
; state 1 or True: Visible
; state 0 or False: Hidden
; state -1 or not specified: Return the current state
;-------------------------------------------------------------------------
Function ksbstate(terrain, state = -1)
	Local layer.KLS, child
	layer = Object.KLS(terrain)
	If layer = Null Then Return -1
	If state > 0 Then
		child = FindChild(layer\entity, "SkyBox")
		If child Then NameEntity(child, "SkyBoxDisabled")
	ElseIf state = 0 Then
		child = FindChild(layer\entity, "SkyBoxDisabled")
		If child Then NameEntity(child, "SkyBox")
	Else
		If FindChild(layer\entity, "SkyBoxDisabled") Then
			Return False
		ElseIf FindChild(layer\entity, "SkyBox")
			Return True
		Else
			Return -1
		EndIf
	EndIf
End Function

; kwpstate(hlayer, state = -1): Set the WaterPlane state of the
; Terrain layer or other layer if WaterPlane has been moved to it.
; state 1 or True: Visible
; state 0 or False: Hidden
; state -1 or not specified: Return the current state
;-------------------------------------------------------------------------
Function kwpstate(terrain, state = -1)
	Local layer.KLS, child
	layer = Object.KLS(terrain)
	If layer = Null Then Return -1
	If state > 0 Then
		child = FindChild(layer\entity, "WaterPlane")
		If child Then NameEntity(child, "WaterPlaneDisabled")
	ElseIf state = 0 Then
		child = FindChild(layer\entity, "WaterPlaneDisabled")
		If child Then NameEntity(child, "WaterPlane")
	Else
		If FindChild(layer\entity, "WaterPlaneDisabled") Then
			Return False
		ElseIf FindChild(layer\entity, "WaterPlane")
			Return True
		Else
			Return -1
		EndIf
	EndIf
End Function

; kwplay(terrain): Return the WaterPlane entity
; Useful to move WaterPlane to another layer, scale, move,
; assign brush, texture or effects to it.
;-------------------------------------------------------------------------
Function kwplay(terrain)
	Local layer.KLS, child
	layer = Object.KLS(terrain)
	If layer = Null Then
		Return 0
	Else
		child = FindChild(layer\entity, "WaterPlane")
		If child = 0 Then child = FindChild(layer\entity, "WaterPlaneDisabled")
		Return child
	EndIf
End Function

; ksblay(terrain): Return the SkyBox entity
; Useful to move SkyBox to another layer, scale, move, etc...
;-------------------------------------------------------------------------
Function ksblay(terrain)
	Local layer.KLS, child
	layer = Object.KLS(terrain)
	If layer = Null Then
		Return 0
	Else
		child = FindChild(layer\entity, "SkyBox")
		If child = 0 Then child = FindChild(layer\entity, "SkyBoxDisabled")
		Return child
	EndIf
End Function

; kRenderLayer(cam, hlayer, clear = False, clearz = False):
; This is the main function to properly render layers and avoid
; alpha incompabilities
; This will render a layer at a time, so you can choose the order
; you want to render your world (usaly the terrain without
; WaterPlane, then the layer containing all your entities, and the
; WaterPlane layer at last
; For the first layer, you must set clear and clearz values to
; True (to clear the background and the ZBuffer), 
; Then False For the others
;-------------------------------------------------------------------------
Function kRenderLayer(cam, hlayer, clear = False, clearz = False)
	Local layer.KLS, item.KLS, n, count, child, flg, wpd, sbd
	layer = Object.KLS(hlayer)
	If layer = Null Then Return -1
	CameraClsMode cam, clear, clearz
	
	;Hide all layers except the one to render
	For item = Each KLS
		If Not(item = layer) Then
			HideEntity item\entity
		EndIf
	Next
	
	;Layer is a KaTE terrain
	If EntityName(layer\entity) = "KaTE terrain" Then
		count = CountChildren(layer\entity)
		For n = 1 To count
			child = GetChild(layer\entity, n)
			HideEntity child
		Next
		
		flg = 0
		For n = 1 To count
			child = GetChild(layer\entity, n)
			If Left$(EntityName(child), 5) = "Layer" Or EntityName(child) = "SkyBox" Or EntityName(child) = "WaterPlane" Then
					ShowEntity child
					RenderWorld
					HideEntity child
					If flg = 0 Then CameraClsMode cam, False, False : flg = 1
			EndIf
		Next
		
		For n = 1 To count
			child = GetChild(layer\entity, n)
			ShowEntity child
		Next
	
	Else ;User layer
		wpd = FindChild(layer\entity, "WaterPlaneDisabled")
		If wpd Then HideEntity wpd
		sbd = FindChild(layer\entity, "SkyBoxDisabled")
		If sbd Then HideEntity sbd
		RenderWorld
		If wpd Then ShowEntity wpd
		If sbd Then  ShowEntity sbd
	EndIf
	
	;Set all layers to visible
	For item = Each KLS
		ShowEntity item\entity
	Next
End Function
;~IDEal Editor Parameters:
;~C#Blitz3D